/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Manual Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <string.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "VarParams.h"

extern int	debugMode;

////////////////////////////////////////////////////////////////////////////
//
// Manual Constructor
//
////////////////////////////////////////////////////////////////////////////

Manual::Manual(int modId)
{
    // fill in id

    id = modId;

    // prepare and fill in the manual title

    char manTitle[TitleSize];
    sprintf(manTitle, "Module         %08d", modId);
    if (debugMode) {
        printf("Manual::Manual(title = %s)\n", manTitle);
    }
    title = manTitle ;

    // prepare and fill in the manual text

    char myText[ManualTextLength];
    sprintf(myText, ManualText, modId);
    int stringLen = strlen(myText);

    text.alloc(ManualSize) ;
    int curChar, strChar;
    curChar = strChar = 0;
    while (curChar < ManualSize-1) {
	text[curChar++] = myText[strChar++];
	if (strChar >= stringLen) { strChar = 0; }
    }
    text[curChar] = '\0';
    textLen = strlen(text);
}


////////////////////////////////////////////////////////////////////////////
//
// Manual searchText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Manual::searchText(char c)
{
    if (debugMode) {
        printf("                    Manual::searchText(title = %s)\n", title);
    }

    // count occurrences of the indicated letter (for busy work)

    int i = 0;
    int count = 0;
    char* mytext = this->text ;
    while (mytext[i] != '\0') {
	if (mytext[i++] == c) { count++; }
    }

    if (debugMode) {
        printf("                    [found %d '%c's among %d characters]\n",
	        		     count, c, i);
    }

    return count;
}

////////////////////////////////////////////////////////////////////////////
//
// Manual firstLast Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Manual::firstLast()
{
    return (text[0] == text[textLen-1]);
}


////////////////////////////////////////////////////////////////////////////
//
// Manual replaceText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Manual::replaceText(char *oldString, char *newString)
{
    if (debugMode) {
        printf("                    Manual::changeText(title = %s)\n", title);
    }

    // check to see if the text starts with the old string

    int oldTextSize = ::strlen(text) + 1;
    int oldStrLength  = ::strlen(oldString);
    int foundMatch = (strncmp(text, oldString, oldStrLength) == 0);

    // if so, change it to start with the new string instead

    if (foundMatch) {

        int newStrLength  = ::strlen(newString);
        int lengthDiff = newStrLength - oldStrLength;
        int newTextSize  = oldTextSize + lengthDiff;

	if (lengthDiff == 0) {
	    strncpy(text, newString, newStrLength);
        } else {
	    char* newText = new char[newTextSize];
	    strncpy(newText, newString, newStrLength);
	    strncpy(&newText[newStrLength], &text[oldStrLength],
		    oldTextSize - oldStrLength);
	    text = newText;
        }
    }

    if (debugMode) {
	if (foundMatch) {
            printf("                    [changed \"%s\" to \"%s\"]\n",
	                                 oldString, newString);
	} else {
            printf("                    [no match, so no change was made]\n");
	}
    }

    if (foundMatch) {
        return 1;
    } else {
        return 0;
    }

}




////////////////////////////////////////////////////////////////////////////
//
// Manual Destructor
//
////////////////////////////////////////////////////////////////////////////

Manual::~Manual()
{
  if (debugMode) {
    printf("Manual::~Manual(title = %s)\n", title);
  }
}

